/////////////////////////////////////////////////////////////////////////
// Schlafzimmer Lichtsteuerung                                         //
//                                                                     //
// Chip type           : ATmega32                                      //
// Clock frequency     : 8,000000 MHz                                  //
// letzte nderung      : 03.10.2007                                   //
/////////////////////////////////////////////////////////////////////////

#include "schlafzi.h"

//#define an 0
//#define aus 1
#define auf 2
#define zu 4

int8_t temp_schlafzi;
int8_t temp_gang_oben;
int8_t temp_schlafzicontroller;

// Variablen fr Uhrzeit/Wartezeit
uint8_t volatile sekunden;
uint8_t volatile minuten;
uint8_t volatile stunden;
uint8_t volatile count;
uint8_t volatile wtag;
uint8_t prescaler;
uint16_t volatile mess_taste_links;
uint16_t volatile mess_taste_rechts;
uint16_t volatile mess_taste_beide;
uint16_t ergebnis_mess_taste_links;
uint16_t ergebnis_mess_taste_rechts;
uint16_t ergebnis_mess_taste_beide;
uint8_t messung_links;
uint8_t messung_rechts;
uint8_t messung_beide;

// Licht Brennzeit
int volatile Gang_Schrankleuchte_Laufzeit_min=0;
int volatile Schlafzi_Schrankleuchte_Laufzeit_min=0;
int volatile Schlafzi_Radiowecker_Laufzeit_min=0;
int volatile Schlafzi_Deckenleuchte_Laufzeit_min=0;
int volatile Schlafzi_Wandleuchte_Laufzeit_min=0;
int volatile Schlafzi_Fernseher_Laufzeit_min=0;

uint8_t S_Gang_Schrank_Tuere=0;

// Buffer
char a_buffer[5];
int i_buffer=0;

// Warteschleife, wegen watchdog max. ca 1800 Millisekunden
void delay_ms(unsigned long millisekunden)
{
    unsigned long i;
    millisekunden *= 1000;
    for (i=0; i < millisekunden; i++)
      _delay_us(1);
} 


/////////////////////////////////////////////////////////////////////////
//Hauptprogramm
int main (void)
/////////////////////////////////////////////////////////////////////////
{

  DDRB = 0x00; // alle Pins von Port B als Eingang
  PORTB = 0xff; // interne Pull-Ups an allen Port-Pins aktivieren
  
  DDRA = 0xff; // alle Pins von Port A als Ausgang
  PORTA = 0x00; // interne Pull-Ups an allen Port-Pins deaktivieren
  
  DDRC = 0xff; // alle Pins von Port C als Ausgang
  PORTC = 0x00; // interne Pull-Ups an allen Port-Pins deaktivieren
  
  // Port D initialization
  // D0=In D1=In D2=In D3=In D4=In D5=Out D6=In D7=In
  DDRB=(1 << DDD5); // Pins D0,D1,D2,D3,D4,D6,D7 auf Eingang, D5 auf Ausgang setzen
  PORTB=0x00; // BIN: 10000000 interne Pull-Ups an allen Port-Pins deaktivieren
  
  // Watchdog auf 2 Sekunden einstellen
  wdt_enable (WDTO_2S);
  
  stunden=5;
  minuten=59;
  sekunden=50;
  wtag=0;
  
  // Interupts einschalten
  sei();
  
  // Alle Relais ausschalten
  PORTA &= ~R_A1_Gang_Schrankleuchte;
  PORTC &= ~R_C0_Schlafzi_Wandleuchte;
  PORTC &= ~R_C1_Schlafzi_Schrankleuchte;
  PORTC &= ~R_C2_Schlafzi_Radiowecker;
  PORTC &= ~R_C6_Schlafzi_Deckenleuchte;
  PORTC &= ~R_C7_Schlafzi_Fernseher; 
  
  // Uhrzeit Timer Interrupt initialisieren
  init_timer_uhr();
  
  S_Gang_Schrank_Tuere=zu;

  /////////////////////////////////////////////////////////////////////////
  // Hauptschleife
  /////////////////////////////////////////////////////////////////////////
  while (1)
    {
      
      // Watchdog zurcksetzten
      wdt_reset();
      
     
	  /////////////////////////////////////////////////////////////////////////
      // Tasten abfragen
	  /////////////////////////////////////////////////////////////////////////
      // Taster Links normal: Wandleuchten an/aus
      // Taster Rechts normal: Deckenleuchte an/aus
      // Taster Links lang: Radio an/aus
      // Taster Rechts lang: Fernseher an/aus
      // Taster Links und Rechts normal: Schrankleuchten an/aus
	  /////////////////////////////////////////////////////////////////////////
      // Messen
	  /////////////////////////////////////////////////////////////////////////
	  if (E_Taster_Schlafzi_rechts==0) 
	    {
		  // Taster Schlafzimmer Rechts wurde gedrckt
          messung_rechts=1;
		  if (ergebnis_mess_taste_rechts < mess_taste_rechts)
		    ergebnis_mess_taste_rechts=mess_taste_rechts;
		}
	  else
	    {
		  // Taster Schlafzimmer Rechts wurde losgelassen
		  mess_taste_rechts=0;
          messung_rechts=0;
		}
	  if (E_Taster_Schlafzi_links==0) 
	    {
		  // Taster Schlafzimmer Links wurde gedrckt
		  messung_links=1;
		  if (ergebnis_mess_taste_links < mess_taste_links)
		    ergebnis_mess_taste_links=mess_taste_links;
		}
	  else
	    {
		  // Taster Schlafzimmer Links wurde losgelassen
		  mess_taste_links=0;
		  messung_links=0;
		}
      if ((E_Taster_Schlafzi_links==0) && (E_Taster_Schlafzi_rechts==0))
	    {
		  // Taster Schlafzimmer Links und Rechts wurden gleichzeitig gedrckt
		  messung_beide=1;
		  if (ergebnis_mess_taste_beide < mess_taste_beide)
		    ergebnis_mess_taste_beide=mess_taste_beide;
		}
	  else
	    {
		  // Taster Schlafzimmer Links oder Rechts wurde losgelassen
		  mess_taste_beide=0;
		  messung_beide=0;
		}  	
     		
	  /////////////////////////////////////////////////////////////////////////
	  //Auswerten
      /////////////////////////////////////////////////////////////////////////
	  if ((messung_rechts==0) && (ergebnis_mess_taste_rechts >= 512) && (ergebnis_mess_taste_beide == 0)) // ab 2 Sekunden -> Fernseher an/aus
	    {
          ergebnis_mess_taste_rechts=0;
		  if (bit_is_set(PINC,PINC7)!=0)             // ist C5 == (+)(=an) ? dann ausmachen
		    {
		      PORTC &= ~R_C7_Schlafzi_Fernseher;       // ausschalten (auf (-) ziehen)
		      Schlafzi_Fernseher_Laufzeit_min=0;
		    }
		  else
		    {
		      PORTC |= R_C7_Schlafzi_Fernseher;          // einschalten (auf (+) ziehen)
		      Schlafzi_Fernseher_Laufzeit_min=240;       // Scherheitsabschaltung nach max. 4 Stunden 
		    }
          delay_ms(200);
		}
	  else if ((messung_rechts==0) && (ergebnis_mess_taste_rechts >= 5) && (ergebnis_mess_taste_beide == 0)) // unter 2 Sekunden -> Deckenleuchte an/aus
	    {
          ergebnis_mess_taste_rechts=0;
		  if (bit_is_set(PINC,PINC6)!=0)             // ist C5 == (+)(=an) ? dann ausmachen
		    {
		      PORTC &= ~R_C6_Schlafzi_Deckenleuchte;       // ausschalten (auf (-) ziehen)
		      Schlafzi_Deckenleuchte_Laufzeit_min=0;
		    }
		  else
		    {
		      PORTC |= R_C6_Schlafzi_Deckenleuchte;          // einschalten (auf (+) ziehen)
		      Schlafzi_Deckenleuchte_Laufzeit_min=120;       // Scherheitsabschaltung der Leuchte nach max. 2 Stunden 
		    }
          delay_ms(200);
		}
      
	  if ((messung_links==0) && (ergebnis_mess_taste_links >= 512) && (ergebnis_mess_taste_beide == 0)) // ab 2 Sekunden -> Radio an/aus
	    {
          ergebnis_mess_taste_links=0;
		  if (bit_is_set(PINC,PINC2)!=0)             // ist C5 == (+)(=an) ? dann ausmachen
		    {
		      PORTC &= ~R_C2_Schlafzi_Radiowecker;       // ausschalten (auf (-) ziehen)
		      Schlafzi_Radiowecker_Laufzeit_min=0;
		    }
		  else
		    {
		      PORTC |= R_C2_Schlafzi_Radiowecker;          // einschalten (auf (+) ziehen)
		      Schlafzi_Radiowecker_Laufzeit_min=240;       // Scherheitsabschaltung nach max. 4 Stunden 
		    }
          delay_ms(200);
		}
	  else if ((messung_links==0) && (ergebnis_mess_taste_links >= 5) && (ergebnis_mess_taste_beide == 0)) // unter 2 Sekunden -> Wandlampen an/aus
	    {
          ergebnis_mess_taste_links=0;
		  if (bit_is_set(PINC,PINC0)!=0)             // ist C5 == (+)(=an) ? dann ausmachen
		    {
		      PORTC &= ~R_C0_Schlafzi_Wandleuchte;       // ausschalten (auf (-) ziehen)
		      Schlafzi_Wandleuchte_Laufzeit_min=0;
		    }
		  else
		    {
		      PORTC |= R_C0_Schlafzi_Wandleuchte;          // einschalten (auf (+) ziehen)
		      Schlafzi_Wandleuchte_Laufzeit_min=300;       // Scherheitsabschaltung der Leuchte nach max. 5 Stunden 
		    }
          delay_ms(200);
		}
		
      if ((messung_beide==0) && (ergebnis_mess_taste_beide >= 5)) // unter 1 Sekunde -> Schranklampen an/aus
	    {
          ergebnis_mess_taste_beide=0;
          ergebnis_mess_taste_rechts=0;
          ergebnis_mess_taste_links=0;
		  if (bit_is_set(PINC,PINC1)!=0)          // ist C5 == (+)(=an) ? dann ausmachen
		    {
		      PORTC &= ~R_C1_Schlafzi_Schrankleuchte;    // ausschalten (auf (-) ziehen)
			  Schlafzi_Schrankleuchte_Laufzeit_min=0;
			}
		  else
		    {
		      PORTC |= R_C1_Schlafzi_Schrankleuchte;   // einschalten (auf (+) ziehen)
			  Schlafzi_Schrankleuchte_Laufzeit_min=180;   // Scherheitsabschaltung der Leuchte nach max. 3 Stunden 
		    }
          delay_ms(200);
		}
	  
      // Sensortaster Gang Schrank, Licht geht an wenn Schrank geffnet wird, und beim schlieen wieder aus oder
      // automatisch nach 3 Minuten wieder aus, auch wenn Schrank nicht wieder geschlossen wird. Licht geht erst 
      //  wieder an, wenn Schrank geschlossen und dann wieder geffnet wird.
      if (E_Taster_Gang_Schrank==0)   // wird geffnet, ist (-)
        {
		  if (S_Gang_Schrank_Tuere==zu) // nur einschalten, wenn Schrank zu war
            {
              S_Gang_Schrank_Tuere=auf;
              PORTA |= R_A1_Gang_Schrankleuchte; // einschalten (auf (+) ziehen)   
              Gang_Schrankleuchte_Laufzeit_min=3;
            }          
        }
      else
        {
          S_Gang_Schrank_Tuere=zu;
          PORTA &= ~R_A1_Gang_Schrankleuchte;    // ausschalten (auf (-) ziehen)
          Gang_Schrankleuchte_Laufzeit_min=0;
        }  
      
      /////////////////////////////////////////////////////////////////////////
      // nach Ablaufzeit Relais ausschalten
      /////////////////////////////////////////////////////////////////////////
      if (Gang_Schrankleuchte_Laufzeit_min==0)
		{
          PORTA &= ~R_A1_Gang_Schrankleuchte;		// ausschalten (auf (-) ziehen)
		}
      if (Schlafzi_Wandleuchte_Laufzeit_min==0)
		{
          PORTC &= ~R_C0_Schlafzi_Wandleuchte;		// ausschalten (auf (-) ziehen)
		}
      if (Schlafzi_Schrankleuchte_Laufzeit_min==0)
		{ 
          PORTC &= ~R_C1_Schlafzi_Schrankleuchte;	// ausschalten (auf (-) ziehen)
	    }
      if (Schlafzi_Radiowecker_Laufzeit_min==0)
		{ 
          PORTC &= ~R_C2_Schlafzi_Radiowecker;		// ausschalten (auf (-) ziehen)
		}
      if (Schlafzi_Deckenleuchte_Laufzeit_min==0)
		{ 
          PORTC &= ~R_C6_Schlafzi_Deckenleuchte;		// ausschalten (auf (-) ziehen)
		}
      if (Schlafzi_Fernseher_Laufzeit_min==0)
		{ 
          PORTC &= ~R_C7_Schlafzi_Fernseher; 		// ausschalten (auf (-) ziehen)
		}
	} // Hauptendlos while Schleife Ende
} // Main Ende
